# This script refers to the tasks described in "C.1 Activities", activities 4 and 5.
install.packages("lavaan", repos = "http://www.da.ugent.be", type = "source") # Install the latest developmental version of lavaan which can handle problematic polychoric close to one (cor_poly between prt10 + prt11 turned out to be close to one).

library(TAM)
library(dplyr)
library(lavaan)

# Data preparation:
blot <- read.csv("BLOT.csv", header = FALSE, sep = ";")
colnames(blot)[1] <- c("id")
colnames(blot)[2:49] <- paste("blot", (1:48), sep ="")

blot_items <- dplyr::select(blot, id, blot1:blot35)

prt <- read.csv("PRTIII.csv", header = FALSE, sep = ";")
colnames(prt)[1] <- c("id")
colnames(prt)[2:14] <- paste("prt", (1:13), sep = "")

df_pre <- merge(blot_items, prt, by = "id") # Merge data
df <- df_pre[,-c(1)] # Drop id column. Not needed for the TAM analysis

################################
# TAM dimensionality analysis: #
################################

# tamaan specification of a two-dimensional Rasch model with correlated factors:
tammodel <- '

LAVAAN MODEL:
F1 =~ 1*blot1__blot35
F2 =~ 1*prt1__prt13


F1 ~~ F1
F2 ~~ F2
F1 ~~ F2

 '

tam_2d <- TAM::tamaan(tammodel, df, control = list(maxiter = 300)) # Fit the model
summary(tam_2d)
IRT.modelfit(tam_2d)

# Fit a *unidimensional* Rasch model using TAM:
fit_rasch_1d_tam <- TAM::tam.mml(df)

# Compare models: See Table 12.16 in ARM4
IRT.compareModels(fit_rasch_1d_tam, tam_2d)


#######################################################
# CFA dimensionality analysis with the lavaan package #
#######################################################

# lavaan: Specifying the Rasch model as a *unidimensional* CFA model. By using 'a' as the same label for all item slopes, slopes will be constrained to be equal across items: 

rasch_model_lavaan <- 'eta =~ a*blot1 + a*blot2 + a*blot3 + a*blot4 + a*blot5 + a*blot6 + a*blot7 + a*blot8 + a*blot9 + a*blot10 + a*blot11 + a*blot12 + a*blot13 + a*blot14 + a*blot15 + a*blot16 + a*blot17 + a*blot18 + a*blot19 + a*blot20 + a*blot21 + a*blot22 + a*blot23 + a*blot24 + a*blot25 + a*blot26 + a*blot27 + a*blot28 + a*blot29 + a*blot30 + a*blot31 + a*blot32 + a*blot33 + a*blot34 + a*blot35 + a*prt1 + a*prt2 + a*prt3 + a*prt4 + a*prt5 + a*prt6 + a*prt7 + a*prt8 + a*prt9 + a*prt10 + a*prt11 + a*prt12 + a*prt13
'

fit_1d_rasch <- lavaan::cfa(rasch_model_lavaan, std.lv = TRUE, data = df, 
                             ordered = TRUE) # Fitting the unidimensional Rasch model

summary(fit_1d_rasch, fit.measures = TRUE, standardized = TRUE)  # Summary of fit and parameter estimates. 

# NOTE (Erratum): Due to a coding error, ARM4, p. 326, reports an incorrect number of degrees of freedom and fit statistics being slightly different from the results obtained from this code ("…chi-square = 1366.043, df = 1126, p < .001, CFI = .880, RMSEA = .038, and SRMR = .176."). The conclusions do not change, though.)


# lavaan: Specifying the Rasch model as a *two-dimensional* CFA model with correlated factors. 
# Notice that using 'a' as the same label for all item slopes of the BLOT constrains the slopes to be equal across items *within* the BLOT factor. Using 'b' as the same label for all item slopes of the PRT constrains the slopes to be equal across items *within* the PRT factor.

two_d_rasch <- 'blot =~ a*blot1 + a*blot2 + a*blot3 + a*blot4 + a*blot5 + a*blot6 + a*blot7 + a*blot8 + a*blot9 + a*blot10 + a*blot11 + a*blot12 + a*blot13 + a*blot14 + a*blot15 + a*blot16 + a*blot17 + a*blot18 + a*blot19 + a*blot20 + a*blot21 + a*blot22 + a*blot23 + a*blot24 + a*blot25 + a*blot26 + a*blot27 + a*blot28 + a*blot29 + a*blot30 + a*blot31 + a*blot32 + a*blot33 + a*blot34 + a*blot35

prt =~ b*prt1 + b*prt2 + b*prt3 + b*prt4 + b*prt5 + b*prt6 + b*prt7 + b*prt8 + b*prt9 + b*prt10 + b*prt11 + b*prt12 + b*prt13
'

fit_2d_rasch <- lavaan::cfa(two_d_rasch, data = df, ordered = TRUE) # Fitting the two-dimensional Rasch model

summary(fit_2d_rasch, standardized = TRUE, fit.measures = TRUE) # Summary of fit and parameter estimates
